/* css.h:       Commercial-Spooling-System / C-Interface
**
** Project:     CSS / Commercial Spooling System
**
** Author:      Unitec Informatik
**              Thomas Madoerin
**              Leerueti 797
**              CH-8625 Gossau, Switzerland
**
**              Fax:        01 935 4466
**              Phone:      01 935 4064  (CH)
**                      0041 1 935 4064  (BRD)
**                     01141 1 935 4064  (USA)
**
** Copyright (c) 2000 by Unitec Informatik
**
** THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE.
** The copyright notice above does not evidence any actual or intended
** publication of such source code.
*/

#ifndef _CSS_H_
#define _CSS_H_

#ifndef EOF
#include <stdio.h>
#endif

#ifdef  __cplusplus
extern  "C" {
#endif

#ifndef TRUE
#define TRUE            1
#define FALSE           0
#endif

/* commands */

#define CS_MORE         0
#define CS_LOCATION     1
#define CS_FIRST        1
#define CS_LAST         2
#define CS_NEXT         3
#define CS_PREV         4
#define CS_STAT         5
#define CS_CLOSE        6
#define CS_OPEN         7
#define CS_KEY          8
#define CS_AKEY         9
#define CS_HOLD         10
#define CS_STOP         11
#define CS_STOP_KEEP    12
#define CS_MOUNT        13
#define CS_GO           14
#define CS_ROUTE        16
#define CS_DELETE       17
#define CS_ACCEL        18
#define CS_TEST_PRINT   19
#define CS_COND_OR      20
#define CS_COND_AND     21
#define CS_COND_SET     22
#define CS_COND_CLEAR   23
#define CS_COND_STORE   24
#define CS_MODIFY       25
#define CS_COUNT        26
#define CS_REC          27
#define CS_NKEY         28
#define CS_REPLOC       29
#define CS_OFFSET       30
#define CS_FROM_BOF     31
#define CS_FROM_EOF     32
#define CS_LOCK         33
#define CS_UNLOCK       34
#define CS_DECEL        35
#define CS_RESCHED      36
#define CS_ACCESS_CHECK 37
#define CS_KILL		38

/* function status-codes */

#define CS_OK           0
#define CS_ERROR        -99
#define CS_EOF          EOF

/* device and report status-codes */

#define CS_READY        '0'
#define CS_STARTING     '1'
#define CS_PRINTING     '2'
#define CS_OFFLINE      '3'
#define CS_BLOCKED      '4'
#define CS_LOCKED       '5'
#define CS_KEEPING      'K'
#define CS_DELETED      'X'
#define CS_ROUTED       'R'

/* short cuts */

#define DEV_DATA(dev)   dev->d
#define FORM_DATA(form) form->d
#define REP_DATA(rep)   rep->d

typedef void *CSS_IOC;          /* opaque IO control structure */

struct  css_forms
{
        char    cssf_form;      /* form-id */
        char    cssf_name[31];  /* form-name / max. strlen = 30 + '\0' */
        char    cssf_pp;        /* page_parity TRUE/FALSE */
        short   cssf_ps;        /* page-size */
        short   cssf_ll;        /* line-length at 10 CPI */
        short   cssf_devnr;     /* may only be mounted on device (0 = any) */
};

struct  css_forminfo
{
        char    cfi_form[4];    /* form-id */
        char    cfi_name[31];   /* form-name / max. strlen = 30 + '\0' */
        char    cfi_pp;         /* page_parity TRUE/FALSE */
        short   cfi_ps;         /* page-size */
        short   cfi_ll;         /* line-length at 10 CPI */
        short   cfi_devnr;      /* may only be mounted on device (0 = any) */
};


typedef struct
{
        short   nr;             /* internal report number */
        long    time;           /* creation time (unix-time) */
}       CSS_REP_ID;


typedef struct
{
        short   numId;          /* numeric device id */
        char    alphaId[17];    /* alphabetic device id */
        char    name[31];       /* printer name */
        char    type[15];       /* printer type */
        char    device[65];     /* device-file */
        char    lockFile[65];   /* external lockfile (el-parameter) */
        char    binMode;        /* TRUE if binary-mode device */
        char    form[4];        /* current form */
        char    status;         /* device status:
                                ** CS_READY:    device ready
                                ** CS_STARTING: starting to print
                                ** CS_PRINTING: printing
                                ** CS_OFFLINE:  device not ready (unused)
                                ** CS_BLOCKED:  held
                                ** CS_LOCKED:   locked (external lock),
                                **              waiting
				** CS_ROUTED:   routed to routeId
                                */
        CSS_REP_ID repId;       /* current report, if CS_PRINTING */
	short	routeId;	/* route device if CS_ROUTED */
#ifndef WINDOWS
        short   uId;            /* user-id of current report */
        short   gId;            /* group-id of current report */
#else
        DWORD   uId;            /* user-id of current report */
        DWORD   gId;            /* group-id of current report */
#endif
        short   nCopys;         /* no. of copys to print */
        short   cCopy;          /* current copy printing */
        long    nPages;         /* no. of pages to print */
        long    cPage;          /* current page printing */
        char    res1[32];       /* reserved for future use */
        long    res2;
        void    *res3;
}       CSS_DEV_DATA;

typedef struct
{
        int     status;
        int     errNo;
        void    *c_;            /* internal data section, don't touch ever !! */
        CSS_DEV_DATA    d;      /* user device data section */
}       CSS_DEV_O_, *CSS_DEV;


typedef struct
{
        char    id[4];
        char    name[31];       /* form-name / max. strlen = 30 + '\0' */
        char    pageParity;     /* page_parity TRUE/FALSE */
        short   pageSize;       /* page-size at 6 LPI */
        short   lineLength;     /* line-length at 10 CPI */
        short   devId;          /* may only be mounted on device (0 = any) */
        char    res1[32];       /* reserved for future use */
        long    res2;
        void    *res3;
}       CSS_FORM_DATA;

typedef struct
{
        int     status;
        int     errNo;
        void    *c_;            /* internal data section, don't touch ever !! */
        CSS_FORM_DATA   d;      /* user form data section */
}       CSS_FORM_O_, *CSS_FORM;


typedef struct
{
        CSS_REP_ID id;          /* report id */
        short   devId;          /* device id /  0 = auto select */
        char    title[31];      /* report-title */
        char    fileName[29];   /* input file name */
        char    form[4];        /* form */
        char    subForm[4];     /* subform / any string */
        char    keepDate[9];    /* keep-date: yyyymmdd */
        char    banner[13];     /* banner-text */
        char    progName[15];   /* program-id */
        char    empId[9];       /* employee-id */
        char    fduOrder[61];   /* fdu-order-info */
        char    fduLang[33];    /* fdu-language */
        char    cons[3];        /* console-no. */
        char    cond[8];        /* conditions */
        char    binMode;        /* TRUE if binary-mode option set */
        short   binMask;        /* binary mode mask */
        char    status;         /* report status:
                                ** CS_READY:    ready  to print
                                ** CS_PRINTING: printing on prDevId
                                ** CS_BLOCKED:  held
                                ** CS_KEEPING:  keeping / printed
                                */
        short   prDevId;        /* printing on device id */
#ifndef WINDOWS
        short   uId;            /* user-id of creator */
        short   gId;            /* group-id of creator */
#else
        DWORD   uId;            /* user-id of creator */
        DWORD   gId;            /* group-id of creator */
#endif
        short   nCopys;         /* no. of copys to print */
        long    nPages;         /* no. of pages to print */
        long    startPage;      /* start-page of current printout */
        long    endPage;        /* end-page of current printout */
        long    stopPage;       /* stop-page of last printout, if stopped */
        short   pageSize;       /* page-size as defined by the lpr P-option */
        short   lineLength;     /* max. line-length */
        short   nFull;          /* no. of full printouts */
        short   nPart;          /* no. of partial printouts */
        short   lDevId;         /* device id of last printout */
#ifndef WINDOWS
        short   lUId;           /* user id of last printout */
#else
        DWORD   lUId;           /* user id of last printout */
#endif
        long    lTime;          /* time of last printout (unix-time) */
        char    res1[28];       /* reserved for future use */
        void    *res3;
}       CSS_REP_DATA;

typedef struct
{
        int     status;
        int     errNo;
        void    *c_;            /* internal data section, don't touch ever !! */
        CSS_REP_DATA    d;      /* user report data section */
}       CSS_REP_O_, *CSS_REP;

typedef struct
{
        short   np;
        short   pp;
}       CSS_REP_ADR;

typedef struct
{
	short	repUid;		/* report modification user */
        short   repVersion;     /* file modification version */
        short   modVersion;     /* report modification version */
        short   modReport;      /* report modified */
        short   insVersion;     /* insert version */
        short   insReport;      /* report inserted */
        short   delVersion;     /* delete version */
        short   delReport;      /* report deleted */
        short   accelVersion;   /* accellerate version */
        short   accelReport;    /* report accellerated */
}       CSS_REP_STAT;

typedef struct
{
        long    adr1;
        short   adr2;
        long    pageNo;
        short   lineNo;
}       CSS_PRINT_ADR;


extern  int     cssStrict;      /* type of error-handling */
extern  int     cssAccess;      /* type of access-checking */

#if defined(__STDC__) && ! defined(NO_PROTOTYPE)
typedef int (*CSS_PRF)(int ch, CSS_PRINT_ADR *cssPrintAdr);
extern  struct css_forms *css_fstat(int form);
extern  struct css_forminfo *css_formstat(char *form);
extern  int CssDevCntl(CSS_DEV cssDev, int cmd, ...);
extern  int CssDevFree(CSS_DEV cssDev);
extern  CSS_DEV CssDevNew(CSS_IOC cssIOC);
extern  int CssDevRead(CSS_DEV cssDev, int cmd, ...);
extern  int CssFormCntl(CSS_FORM cssForm, int cmd, ...);
extern  int CssFormFree(CSS_FORM cssForm);
extern  CSS_FORM CssFormNew(CSS_IOC cssIOC);
extern  int CssFormRead(CSS_FORM cssForm, int cmd, ...);
extern  CSS_IOC CssIOCNew();
extern  void CssIOCFree(CSS_IOC cssIOC);
extern  int CssRepCntl(CSS_REP cssRep, int cmd, ...);
extern  int CssRepFree(CSS_REP cssRep);
extern  CSS_REP CssRepNew(CSS_IOC cssIOC);
extern  int CssRepRead(CSS_REP cssRep, int cmd, ...);
extern  int CssPrint(CSS_REP cssRep, CSS_DEV cssDev, CSS_PRF PrFunc);
extern  int CssRepAdd(CSS_REP cssRep, char *title, char *form, int pages);
#else
typedef int (*CSS_PRF)();
extern  struct css_forms *css_fstat();
extern  struct css_forminfo *css_formstat();
extern  int CssDevCntl();
extern  int CssDevFree();
extern  CSS_DEV CssDevNew();
extern  int CssDevRead();
extern  int CssFormCntl();
extern  int CssFormFree();
extern  CSS_FORM CssFormNew();
extern  int CssFormRead();
extern  CSS_IOC CssIOCNew();
extern  void CssIOCFree();
extern  int CssRepCntl();
extern  int CssRepFree();
extern  CSS_REP CssRepNew();
extern  int CssRepRead();
extern  int CssPrint();
extern  int CssRepAdd();
#endif

#ifdef  __cplusplus
}
#endif

#endif

/* end css.h */
